package com.mangst.sat;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

public class ExpressionTest
{
  private final static List<Boolean> values = Arrays.asList( new Boolean[] { true, false, true, true } );
  private final static List<String> variables = Arrays.asList( new String[] { "a", "b", "c", "d" } );

  @Test
  public void testBadInputs() throws Exception
  {
    Expression expression;
    String formulas[] = { "!", "a &", "& a", "! & a", "a & !", "(a & b", "a b", "a !" };
    for ( String f : formulas )
    {
      try
      {
        expression = new Expression( variables, f );
        expression.eval( values );
        Assert.fail( f );
      }
      catch ( SATException e )
      {
      }
    }
  }

  @Test
  public void test() throws Exception
  {
    Expression expression;

    expression = new Expression( variables, "a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "(a)" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "((a))" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "(!a)" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!(a)" );
    Assert.assertEquals( false, expression.eval( values ) );

    //same variable AND

    expression = new Expression( variables, "a & a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "a & !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!a & a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!a & !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    //different variable AND 

    expression = new Expression( variables, "a & b" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "a & !b" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!a & b" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!a & !b" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "b & a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "b & !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!b & a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!b & !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    //same variable OR

    expression = new Expression( variables, "a | a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "a | !a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!a | a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!a | !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    //different variable OR 

    expression = new Expression( variables, "a | b" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "a | !b" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!a | b" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!a | !b" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "b | a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "b | !a" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!b | a" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "!b | !a" );
    Assert.assertEquals( true, expression.eval( values ) );

    //NOTing a sub expression

    expression = new Expression( variables, "!(a & b)" );
    Assert.assertEquals( true, expression.eval( values ) );

    //sub expression

    expression = new Expression( variables, "(a | !b) & c" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "(a | !b) & !c" );
    Assert.assertEquals( false, expression.eval( values ) );

    expression = new Expression( variables, "!(a | !b) & c" );
    Assert.assertEquals( false, expression.eval( values ) );

    //sub-sub expression

    expression = new Expression( variables, "((a & !b) | c) & d" );
    Assert.assertEquals( true, expression.eval( values ) );

    expression = new Expression( variables, "((a & !b) | c) & !d" );
    Assert.assertEquals( false, expression.eval( values ) );
  }

  @Test
  public void testToString() throws Exception
  {
    Expression expression;

    //extra spaces
    expression = new Expression( variables, "  a  " );
    Assert.assertEquals( "a", expression.toString() );

    //extra spaces with NOT
    expression = new Expression( variables, " ! a  " );
    Assert.assertEquals( "!a", expression.toString() );

    //NOT applied to a sub-expression with extra spaces
    expression = new Expression( variables, " ! (a & b)" );
    Assert.assertEquals( "!(a & b)", expression.toString() );

    //redundent parens, extra spaces
    expression = new Expression( variables, "((((a & !  b)  |  c) & (d)))" );
    Assert.assertEquals( "((a & !b) | c) & d", expression.toString() );
  }
}
