package com.mangst.sat;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;

public class SATSolverTest
{
  @Test
  public void testCombos() throws Exception
  {
    List<List<Boolean>> list;
    
    //1
    list = SATSolver.combos(1);
    Assert.assertEquals( ( int ) Math.pow( 2, 1 ), list.size() );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true } ) ) );
    
    //2
    list = SATSolver.combos(2);
    Assert.assertEquals( ( int ) Math.pow( 2, 2 ), list.size() );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, true } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, true } ) ) );
    
    //3
    list = SATSolver.combos(3);
    Assert.assertEquals( ( int ) Math.pow( 2, 3 ), list.size() );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, false, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, false, true } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, true, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { false, true, true } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, false, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, false, true } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, true, false } ) ) );
    Assert.assertTrue( list.toString(), list.contains( Arrays.asList( new Boolean[] { true, true, true } ) ) );
    
    //0
    try
    {
      SATSolver.combos(0);
      Assert.fail();
    }
    catch (SATException e)
    {
    }
    
    //-1
    try
    {
      SATSolver.combos(-1);
      Assert.fail();
    }
    catch (SATException e)
    {
    }
    
  }

  @Test
  public void testGetVariables()
  {
    List<String> variables;
    variables = SATSolver.getVariables( "(a | !b) & c" );
    Assert.assertEquals( variables.toString(), 3, variables.size() );
    Assert.assertTrue( variables.contains( "a" ) );
    Assert.assertTrue( variables.contains( "b" ) );
    Assert.assertTrue( variables.contains( "c" ) );

    variables = SATSolver.getVariables( "a & a" );
    Assert.assertEquals( variables.toString(), 1, variables.size() );
    Assert.assertTrue( variables.contains( "a" ) );
    
    //case sensitive
    variables = SATSolver.getVariables( "a & A" );
    Assert.assertEquals( variables.toString(), 2, variables.size() );
    Assert.assertTrue( variables.contains( "a" ) );
    Assert.assertTrue( variables.contains( "A" ) );
  }

  @Test
  public void testSolve() throws Exception
  {
    List<Solution> solutions;
    Map<String, Boolean> values = new HashMap<String, Boolean>();

    //1
    solutions = SATSolver.solve( "( (a | ! b\n)   \t & (  c))" ); //extra parens, extra whitespace
    Assert.assertEquals( solutions.toString(), 3, solutions.size() );

    values.clear();
    values.put( "a", true );
    values.put( "b", true );
    values.put( "c", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", true );
    values.put( "b", false );
    values.put( "c", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", false );
    values.put( "b", false );
    values.put( "c", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    //2
    solutions = SATSolver.solve( "(!a | !b) & ( !a | b | !c) & (!a | c | !d) & (a | d)" );
    Assert.assertEquals( solutions.toString(), 5, solutions.size() );

    values.clear();
    values.put( "a", true );
    values.put( "b", false );
    values.put( "c", false );
    values.put( "d", false );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", false );
    values.put( "b", true );
    values.put( "c", true );
    values.put( "d", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", false );
    values.put( "b", true );
    values.put( "c", false );
    values.put( "d", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", false );
    values.put( "b", false );
    values.put( "c", true );
    values.put( "d", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );

    values.clear();
    values.put( "a", false );
    values.put( "b", false );
    values.put( "c", false );
    values.put( "d", true );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );
    
    //3
    solutions = SATSolver.solve( "!(a | b)" );
    Assert.assertEquals( solutions.toString(), 1, solutions.size() );

    values.clear();
    values.put( "a", false );
    values.put( "b", false );
    Assert.assertTrue( solutions.contains( new Solution( values ) ) );
  }
}
